import json
import subprocess
from typing import List


class Singleton(type):
    _instances = {}

    def __call__(cls, *args, **kwargs):
        if cls not in cls._instances:
            cls._instances[cls] = super(Singleton, cls).__call__(*args, **kwargs)
        return cls._instances[cls]


def generate_dagster_name(string) -> str:
    """
    Generate a dagster safe name (^[A-Za-z0-9_]+$.)
    """
    return string.replace("-", "_").replace(" ", "_").replace(":", "_")


def generate_dbt_group_name(node_info):
    if len(node_info.get("fqn", [])) >= 3:
        return "_".join(node_info["fqn"][1:-1])

    return "dbt"


# TODO: Refactor to different file
def run_cli(commands: List[str], project_dir: str, meltano_bin: str = "meltano"):
    return json.loads(
        subprocess.run(
            [meltano_bin] + commands,
            cwd=project_dir,
            stdout=subprocess.PIPE,
            universal_newlines=True,
            check=True,
        ).stdout
    )



import json
import logging
import os
import subprocess

from dagster import job, op, repository, with_resources
from dagster._utils import file_relative_path
from dagster_dbt import dbt_cli_resource
from dagster_meltano import load_jobs_from_meltano_project

MELTANO_PROJECT_ROOT = os.getenv("MELTANO_PROJECT_ROOT", os.getcwd())
MELTANO_BIN = os.getenv("MELTANO_BIN", "meltano")

meltano_jobs = load_jobs_from_meltano_project(MELTANO_PROJECT_ROOT)


@repository
def repository():
    return [meltano_jobs]


# MELTANO_PROJECT_DIR = file_relative_path(
#     __file__,
#     "../",
# )
# DBT_PROJECT_DIR = file_relative_path(
#     __file__,
#     "../transform",
# )
# DBT_PROFILES_DIR = file_relative_path(
#     __file__,
#     "../transform/profiles",
# )
# DBT_EXECUTABLE = file_relative_path(
#     __file__,
#     "../.meltano/transformers/dbt-postgres/venv/bin/dbt",
# )
# DBT_TARGET_PATH = "/workspace/meltano/transform/target"


# @repository
# def dagster_meltano():
#     return [
#         load_jobs_from_meltano_project("/workspace/meltano"),
#         with_resources(
#             load_assets_from_meltano_project(
#                 meltano_project_dir=MELTANO_PROJECT_DIR,
#                 dbt_project_dir=DBT_PROJECT_DIR,
#                 dbt_profiles_dir=DBT_PROFILES_DIR,
#                 # dbt_target_dir=DBT_TARGET_PATH,
#                 dbt_use_build_command=False,
#             ),
#             {
#                 "meltano": meltano_resource,
#                 "dbt": dbt_cli_resource.configured(
#                     {
#                         "project_dir": DBT_PROJECT_DIR,
#                         "profiles_dir": DBT_PROFILES_DIR,
#                         # "target_path": DBT_TARGET_PATH,
#                         "dbt_executable": "/dagster-testing/.meltano/transformers/dbt-postgres/venv/bin/dbt",
#                     },
#                 ),
#             },
#         ),
#     ]



import json
import logging
import subprocess
from typing import List, Optional, Union

from dagster import JobDefinition, ScheduleDefinition

from dagster_meltano.meltano_resource import MeltanoResource


def load_jobs_from_meltano_project(
    project_dir: Optional[str],
) -> List[Union[JobDefinition, ScheduleDefinition]]:
    """This function generates dagster jobs for all jobs defined in the Meltano project. If there are schedules connected
    to the jobs, it also returns those.

    Args:
        project_dir (Optional[str], optional): The location of the Meltano project. Defaults to os.getenv("MELTANO_PROJECT_ROOT").

    Returns:
        List[Union[JobDefinition, ScheduleDefinition]]: Returns a list of either Dagster JobDefinitions or ScheduleDefinitions
    """
    meltano_resource = MeltanoResource(project_dir)
    meltano_jobs = meltano_resource.jobs

    return list(meltano_jobs)


if __name__ == "__main__":
    load_jobs_from_meltano_project("/workspace/meltano")
